Node.js - phplike
=======================

Chinese Readme: https://github.com/puritys/nodejs-phplike/wiki/%E4%B8%AD%E6%96%87%E7%89%88-Readme


This project's purpose is to provider some PHP synchronous functions for the web development on Node.js.

Node.js is a event driven language and it has many asynchronous I/O methods. Asynchronous functions is not a bad way for a web system, it helps us to develop a non-blocking I/O program. But sometimes, we want to make code readable and easier to be maintained that asynchronous functions don't have them.

In order to reduce the number of callback functions on Node.js scripts and improve the readability. I create the library phplike which supports many synchronous functions for Node.js such as "exec", "curl", "fsockopen" that can be executed without complicated callback functions. In additional, phplike provides the function cUrl which has the same operations with PHP's function called curl. It will let you make an HTTP request synchronously. 


* npm: https://npmjs.org/package/phplike
* All version: https://registry.npmjs.org/phplike
* Git Source: https://github.com/puritys/nodejs-phplike


Travis CI status: [![Unit testing](https://travis-ci.org/puritys/nodejs-phplike.png?branch=master)](https://travis-ci.org/puritys/nodejs-phplike) [![Coverage Status](https://coveralls.io/repos/puritys/nodejs-phplike/badge.png?branch=master)](https://coveralls.io/r/puritys/nodejs-phplike?branch=master)

<img src="https://camo.githubusercontent.com/fe2d9e9063dabaf5951ef8f3835bbbc16cec52e3/68747470733a2f2f706f7365722e707567782e6f72672f7a6f72646975732f6c696768746e63616e64792f6c6963656e73652e737667" alt="license">


## Install phplike

* sudo npm install -g phplike

If your computer is not a normal OS, and it doesn't have the header files of Curl, You will need to install node-gyp first. The installation of the phplike library will compile the C/C++ code with node-gyp. Usually, NPM will automatically install node-gyp when you try to install the phplike. Or you can install node-gyp by yourself.

* sudo npm install -g node-gyp

```
   var php = require("phplike/module");
   var dirs = php.exec("ls ./");
   php.print_r(dirs);
```

How many OS does phplike support?
---------------------------
* Linux: Every version of the phplike support Linux systems.
* Mac:  Supported from phplike@2.0.5  to  latest 
* Windows: Only the following version are supported: phplike@2.1.0 、 phplike@2.2.8 , phplike@2.4.4 . I have tested features of the phplike on windows 8 and windows XP.
* Raspberry PI (Pidora OS with ARM CPU): phplike@2.2.2 ~ Latest

| OS       | Suggested phplike Version |
|----------|:-------------:|
| Linux    | Latest |
| Mac      | 2.0.5  ~ latest |
| Windows  | 2.2.8, 2.4.2 |
| Raspberry PI | 2.2.2 ~ latest |

If you have any issue for installing phplike, please open a issue in anytime. I will be pleasant to help you.
 
Dependency
-----------
* Phplike have been already tested in Node.js version from 0.10.x to 0.12.x and io.js 1.0.0 to 2.1.0. Here is the test report: https://travis-ci.org/puritys/nodejs-phplike 
* libcurl (libcurl-7.19) : Linux system already has this built-in package. Please install libcurl-devel :  sudo yum install  libcurl-devel
* python 2.4 ~ :  phplike use node-gyp to compile C/C++ codes. It needs python with the version must be bigger than 2.7.8 , you can download python from here https://www.python.org/downloads/.

After the new version of phplike 2.2.0, I committed all binary files which already compiled in Windows, Mac and Linux, you can just install the phplike without compiling C/C++ now.


Completed PHP Method
-------------------------

**System function**

* <a href="https://github.com/puritys/nodejs-phplike/wiki/function-exec">exec</a> : Execute an external program
* system
* <a href="https://github.com/puritys/nodejs-phplike/wiki/Curl">curl</a> : curl_init, curl_setopt, curl_exec, curl_close, reuqest ( HTTP sync )
* usleep , sleep

**basic**
* print_r
* is_string
* is_int
* is_object
* is_array
* is_numeric
* is_int
* isset
* empty
* exit
* explode
* implode

**File Handler**
* file_get_contents
* file_put_contents
* mkdir
* unlink
* rmdir (dirname, isForce)
* is_file, is_dir
* readdir (get all file list in select directory)

**Encoder and Decoder**

* json_encode, json_decode, handle multibyte: json_decode(xx, 'JSON_UNESCAPED_UNICODE')
* md5
* base64_encode
* base64_decode

**String**

* sprintf
* str_pad
* substr (string, start, length)
* strtolower
* strtoupper

**XML Parser**
<a href="https://github.com/puritys/nodejs-phplike/wiki/XML-Parser">XML Parser Document</a>

* DOMDocument
 * getElementsByTagName

* DOMElement
 * firstChild
 * lastChild
 * hasAttributes

**Socket**

* fsockopen
* sendcmd
* fwrite
* fread

**<a href="https://github.com/puritys/nodejs-phplike/wiki/Mysql-in-Node.js">Mysql</a>**

* mysqli_connect
* mysql_connect
* mysql_select_db
* mysql_query
* mysql_close
* mysql_create_db
* mysql_insert_id

**array**

* shuffle
* array_merge
* array_rand

**Others**

* time, date, mktime
* chr, ord : string to ascii number, ascii number to string
* decbin, bindec
* parse_str : parse "a=b&c=d" to {"a": "b", "c": "d"}
* clone:  clone a object or array
* getcwd
* urlencode, urldecode
* intval: convert string to integer
* strval: convert integer to string
* trim
* http_build_query




Execute phplike in global mode sample
-------------------------------

You can directly use phplike functions. Functions of phplike are defined at the global object when you require the index.js. It means that you don't need the prefix before calling the phplike's function. The Node.js coding will be more like PHP's. The only one disadvantage is that defining a function at the global object is easier to meet the conflict problem.

exec(command, printInScreen = true); 

<pre>
    require("phplike");
    var tm = time();
    sleep(10);
    var result = exec("ls -la");
    print_r(result);
</pre>

Execute phplike in module mode sample (phplike 2.0)
----------------------------------------------


You can require the module.js of phplike, it will return the Phplike object. This object includes many functions of PHP for you to use at any time. The module mode will not change methods of the global object so it won't be a conflict with other Node.js native functions.

```
    var php = require("phplike/module.js");
    var tm = php.time();
    php.sleep(10);
    var result = php.exec("ls -la");
    php.print_r(result);
```


Example code for php curl 
------------------------------
```
    require('phplike');
    
    var url = "https://www.google.com.tw/search?q=php+unit+test";
    var header = {"Cookie": "xxx"};
    
    var c = curl_init();
    curl_setopt(c, 'CURLOPT_URL', url);
    var res = curl_exec(c);
    
    curl_close(c);
    
    console.log("respones = " + res);
```

Example code for php post (Using module mode)
------------------------------

```
    var php = require("phplike/module.js");
    var url = "http://localhost:8080/";
    var param = {"q": "x"};
    var header = {"Cookie": "xxx"};
    var c = php.curl_init();
    php.curl_setopt(c, 'CURLOPT_URL', url);
    php.curl_setopt(c, 'CURLOPT_POST', 1);
    php.curl_setopt(c, 'CURLOPT_POSTFIELDS', "a=bbb&c=eee");
    php.curl_setopt(c, 'CURLOPT_HTTPHEADER', header);
    var res = php.curl_exec(c);
    var responseHeader = php.getResponseHeader(); // Get header
```

Example code for making a blocking request 
------------------------------------------
```
    var phplikeMod = require('phplike/module.js');

    var url = "http://localhost:8080/";
    var param = {"q": "x"};
    var header = {"Cookie": "xxx"};
    var res = phplikeMod.request("GET", url, param, header);
```

Functions will be implemented in the future
------------------------------------------
* abs
* acos

phplike Development
------------------
* Visual studio: Windows will need this software in order to compile C/C++ code,  you can download from here  http://www.visualstudio.com/zh-tw/downloads/download-visual-studio-vs#DownloadFamilies_4 .  Notice! If you install vs2010 Express, it only support 32bit, so you should install the 32bit version Node.js too.
 




